module timer_main(
  // Clock Input (50 MHz)
  input  CLOCK_50,
  //  Push Buttons
  input  [3:0]  KEY,
  //  DPDT Switches 
  input  [17:0]  SW,
  //  7-SEG Displays
  output  [6:0]  HEX0, HEX1, HEX2, HEX3, HEX4, HEX5, HEX6, HEX7,
  //  LEDs
  output  [8:0]  LEDG,  //  LED Green[8:0]
  output  [17:0]  LEDR,  //  LED Red[17:0]
  //  GPIO Connections
  inout  [35:0]  GPIO_0, GPIO_1
);

//  set all inout ports to tri-state
assign  GPIO_0    =  36'hzzzzzzzzz;
//assign  GPIO_1    =  36'hzzzzzzzzz;

assign GPIO_1[35:32] = 4'hz;
assign GPIO_1[31:0] = final_count;

wire [6:0] myclock;
wire RST;
assign RST = KEY[0];

// Setup clock divider
clock_divider cdiv(CLOCK_50,RST,myclock);
assign GPIO_0[6:0] = myclock;

// Connect interesting signals to green LEDs
assign LEDG[0] = timer_clock;
assign LEDG[1] = trigger;

// Connect dip switches to red LEDS
assign LEDR[17:0] = SW[17:0];

// set up counters
wire [3:0] digit7, digit6, digit5, digit4, digit3, digit2, digit1, digit0;
wire ovr0, ovr1, ovr2, ovr3, ovr4, ovr5, ovr6, ovr7;

wire reset = ~postrig & KEY[0];
decimal_counter count0(digit0,ovr0,counter_clock,reset);
decimal_counter count1(digit1,ovr1,ovr0,reset);
decimal_counter count2(digit2,ovr2,ovr1,reset);
decimal_counter count3(digit3,ovr3,ovr2,reset);
decimal_counter count4(digit4,ovr4,ovr3,reset);
decimal_counter count5(digit5,ovr5,ovr4,reset);
decimal_counter count6(digit6,ovr6,ovr5,reset);
decimal_counter count7(digit7,ovr7,ovr6,reset);

// map to 7-segment displays

hex_7seg dsp0(digit0,HEX0);
hex_7seg dsp1(digit1,HEX1);
hex_7seg dsp2(digit2,HEX2);
hex_7seg dsp3(digit3,HEX3);
hex_7seg dsp4(digit4,HEX4);
hex_7seg dsp5(digit5,HEX5);
hex_7seg dsp6(digit6,HEX6);
hex_7seg dsp7(digit7,HEX7);

reg [31:0] count;
reg [31:0] final_count;

reg  state;
parameter
  HOLD = 1'b0,
  COUNT = 1'b1;

wire timer_clock = myclock[2];

wire counter_clock;
assign counter_clock = (state==COUNT? timer_clock: 1'b0);
wire trigger = ~KEY[3];

reg oldtrig;
wire postrig = (trigger & ~oldtrig);
wire negtrig = (~trigger & oldtrig);


always @ (posedge timer_clock or negedge RST)
begin
	if (!RST) state <= HOLD;
	else 
	begin
	oldtrig <= trigger;
	case (state)
	HOLD : if (postrig) 
		begin
		  state <= COUNT;
		  count <= 1;
		end
	COUNT: if (negtrig)
		begin
		   state <= HOLD;
		   final_count <= count;
		end
		else count <= count + 1'b1;
	endcase
	end
end


endmodule